/*
* Claude EMERY 07/03/2025
* Affichage Date/Heure depuis serveur NTP & température/humidité
* Sur afficheur Oled 0.96" avec Adafruit_SSD1306.h
* ESP32-Wroom + Oled 0.96 + dht22
* DHT22 SDA en D5
* Oled 0.96 SSD1306 SDA en D21/SCL en D22
*/

// Librairies 
#include <WiFi.h>               // Connexion Wifi
#include <Wire.h>               // Gestion I2C pour Oled SDA / SCL
#include <Adafruit_GFX.h>       // Affichage graphique
#include <Adafruit_SSD1306.h>   // Driver Oled 0.96" - SSD1306
#include <Adafruit_Sensor.h>    // Sensor Adafruit
#include <DHT.h>                // DHT22-DHT11
#include <NTPClient.h>          // Client NTP - Obtenir Année, Jour, Heure, Minute, Seconde, etc ....
#include <WiFiUdp.h>            // Utilisation du protocole UDP

// Configuration WiFi
const char* ssid = "Votre_SSID";                  // Votre_SSID
const char* password = "Votre_Mot_de_Passe";      // Votre_Mot_de_Passe

// Configuration OLED
#define SCREEN_WIDTH 128
#define SCREEN_HEIGHT 64
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, -1);

#define DHTPIN 5                // DHT22 en D5
//#define DHTTYPE    DHT11      // DHT 11
#define DHTTYPE    DHT22        // DHT 22 (AM2302)
//#define DHTTYPE    DHT21      // DHT 21 (AM2301)

DHT dht(DHTPIN, DHTTYPE);       // Initialisation du DHT

// Configuration NTP - Serveur NTP pool.ntp.org
WiFiUDP ntpUDP;
NTPClient timeClient(ntpUDP, "pool.ntp.org", 3600, 60000);    // UTC+1 - Recupération des données Date & Heure - Paris

void setup() {
    Serial.begin(115200);

    // Demarrage du DHT
    dht.begin();  
    
    // Connexion WiFi
    WiFi.begin(ssid, password);
    while (WiFi.status() != WL_CONNECTED) {
        delay(500);
        Serial.print(".");
    }
    Serial.println("\nWiFi connecté.");

    // Initialisation de l'écran OLED
    if (!display.begin(SSD1306_SWITCHCAPVCC, 0x3C)) {
        Serial.println("Échec de l'initialisation de l'écran OLED !");
        for (;;);
    }
    // Affichage Wifi connecté sur Oled
    display.clearDisplay();
    display.setTextSize(1);
    display.setTextColor(WHITE);
    display.setCursor(15, 0);             // Colonne 15  Ligne 10
    display.print("WiFi connect");
    display.cp437(true);                  // Affichage du é - code cp437 = 130
    display.write(130);
;   display.display();
    delay(1000);                          // Mise à jour chaque seconde
   
    // Démarrer le client NTP
    timeClient.begin();  
}

void loop() {
    timeClient.update();                // Update de la date & heure

    float t = dht.readTemperature();    // Lecture temperature
    float h = dht.readHumidity();       // Lecture humidité

    if (isnan(h) || isnan(t)) {                               // Si h ou t sont faux (isnan) = affichage message d'erreur
      Serial.println("Failed to read from DHT sensor!");      // Detection erreur du DHT
  }

    // Récupérer et formater l'heure et la date
    unsigned long epochTime = timeClient.getEpochTime();
    struct tm *timeInfo;
    time_t rawTime = epochTime;
    timeInfo = localtime(&rawTime);
    int day = timeInfo->tm_mday;
    int month = timeInfo->tm_mon + 1;       // Janvier = 0
    int year = timeInfo->tm_year + 1900;
    int hours = timeInfo->tm_hour;
    int minutes = timeInfo->tm_min;
    int seconds = timeInfo->tm_sec;

    // Création des variables dateBuffer & timeBuffer
    char dateBuffer[20];
    sprintf(dateBuffer, "%02d/%02d/%04d", day, month, year);          // Format jj/mm/aaaa
    char timeBuffer[10];
    sprintf(timeBuffer, "%02d:%02d:%02d", hours, minutes, seconds);   //Format hh:mm:ss

    // Effacer l'écran et définition de la taille des caractéres
    display.clearDisplay();                  // Effacement de l'afficheur
    display.setTextSize(1);                  // Taille 1 pour tous les caractéres - La taille de change pas - Il n'est pas nécessaire de la redefinir dans le code 

    // Affichage 1 ligne
    display.setCursor(0,0);                  // Position du curseur - colonne 0 et ligne 0
    display.print("---------------------");  // 21 - seulement

    // Date
    display.setCursor(15, 7);                // Colonne 15  Ligne 7
    display.print("Date: ");
    display.print(dateBuffer);               // Affichage de la date au format jj/mm/aaaa
    // Heure
    display.setCursor(18, 17);               // Colonne 18  Ligne 17
    display.print("Heure: ");
    display.print(timeBuffer);               // Affichage de l'heure au format hh:mm:ss

    // Affichage 1 ligne
    display.setCursor(0,23);                 // Position du curseur - colonne 0 et ligne 23
    display.print("---------------------");  // 21 - seulement

    // Preparation de l'affichage température
    display.setCursor(0,35);                 // Position du curseur - colonne 0 et ligne 35
    display.print("Temp");
    display.cp437(true);
    display.write(130);                      // Caractere é
    display.print("rature: ");
    display.setCursor(75,35);                // Position du curseur - colonne 75 et ligne 35
    display.print(t);                        // Affichage du contenu de la variable t
    display.print(" ");
    display.cp437(true);                     // Driver cp437 pour ° Signe de temperature Celcius
    display.write(167);                      // code cp437 167 = ° 
    display.print("C");
 
    // Preparation de l'affichage humidité
    display.setCursor(0, 46);                // Position du curseur - colonne 0 et ligne 46
    display.print("Humidit");
    display.cp437(true);
    display.write(130);                      // Caractere é
    display.print(": ");
    display.setCursor(75, 46);               // Position du curseur - colonne 75 et ligne 46
    display.print(h);                        // Affichage du contenu de la variable h
    display.print(" %"); 

    // Affichage 1 ligne - Limite max du bas de l'Oled = 60
    display.setCursor(0,60);                 // Position du curseur - colonne 0 et ligne 60
    display.print("---------------------");  // 21 - seulement

    display.display();
    delay(1000);                             // Mise à jour chaque seconde
}